@@ -1,7 +1,7 @@
 from typing import Any, ClassVar, Dict, Generic, Tuple, Type, TypeVar, Union, get_type_hints
 
-from pydantic import BaseModel, create_model
-from pydantic.class_validators import gather_validators
+from .class_validators import gather_all_validators
+from .main import BaseModel, create_model
 
 _generic_types_cache: Dict[Tuple[Type[Any], Union[Any, Tuple[Any, ...]]], Type[BaseModel]] = {}
 GenericModelT = TypeVar('GenericModelT', bound='GenericModel')
@@ -17,9 +17,8 @@ class GenericModel(BaseModel):
         else:
             raise TypeError(f'Type {cls.__name__} cannot be used without generic parameters, e.g. {cls.__name__}[T]')
 
-    def __class_getitem__(  # type: ignore
-        cls: Type[GenericModelT], params: Union[Type[Any], Tuple[Type[Any], ...]]
-    ) -> Type[BaseModel]:
+    # Setting the return type as Type[Any] instead of Type[BaseModel] prevents PyCharm warnings
+    def __class_getitem__(cls: Type[GenericModelT], params: Union[Type[Any], Tuple[Type[Any], ...]]) -> Type[Any]:
         cached = _generic_types_cache.get((cls, params))
         if cached is not None:
             return cached
@@ -27,7 +26,7 @@ class GenericModel(BaseModel):
             raise TypeError('Cannot parameterize a concrete instantiation of a generic model')
         if not isinstance(params, tuple):
             params = (params,)
-        if any(isinstance(param, TypeVar) for param in params):  # type: ignore
+        if cls is GenericModel and any(isinstance(param, TypeVar) for param in params):  # type: ignore
             raise TypeError(f'Type parameters should be placed on typing.Generic, not GenericModel')
         if Generic not in cls.__bases__:
             raise TypeError(f'Type {cls.__name__} must inherit from typing.Generic before being parameterized')
@@ -40,10 +39,10 @@ class GenericModel(BaseModel):
             k: resolve_type_hint(v, typevars_map) for k, v in instance_type_hints.items()
         }
 
-        model_name = concrete_name(cls, params)
-        validators = gather_validators(cls)
+        model_name = cls.__concrete_name__(params)
+        validators = gather_all_validators(cls)
         fields: Dict[str, Tuple[Type[Any], Any]] = {
-            k: (v, cls.__fields__[k].schema) for k, v in concrete_type_hints.items() if k in cls.__fields__
+            k: (v, cls.__fields__[k].field_info) for k, v in concrete_type_hints.items() if k in cls.__fields__
         }
         created_model = create_model(
             model_name=model_name,
@@ -60,11 +59,14 @@ class GenericModel(BaseModel):
             _generic_types_cache[(cls, params[0])] = created_model
         return created_model
 
-
-def concrete_name(cls: Type[Any], params: Tuple[Type[Any], ...]) -> str:
-    param_names = [param.__name__ if hasattr(param, '__name__') else str(param) for param in params]
-    params_component = ', '.join(param_names)
-    return f'{cls.__name__}[{params_component}]'
+    @classmethod
+    def __concrete_name__(cls: Type[Any], params: Tuple[Type[Any], ...]) -> str:
+        """
+        This method can be overridden to achieve a custom naming scheme for GenericModels
+        """
+        param_names = [param.__name__ if hasattr(param, '__name__') else str(param) for param in params]
+        params_component = ', '.join(param_names)
+        return f'{cls.__name__}[{params_component}]'
 
 
 def resolve_type_hint(type_: Any, typevars_map: Dict[Any, Any]) -> Type[Any]:
