@@ -1,4 +1,5 @@
 import datetime
+from dataclasses import asdict, is_dataclass
 from decimal import Decimal
 from enum import Enum
 from ipaddress import IPv4Address, IPv4Interface, IPv4Network, IPv6Address, IPv6Interface, IPv6Network
@@ -7,7 +8,8 @@ from types import GeneratorType
 from typing import Any, Callable, Dict, Type, Union
 from uuid import UUID
 
-from pydantic.types import SecretBytes, SecretStr
+from .color import Color
+from .types import SecretBytes, SecretStr
 
 __all__ = 'pydantic_encoder', 'custom_pydantic_encoder', 'timedelta_isoformat'
 
@@ -17,14 +19,15 @@ def isoformat(o: Union[datetime.date, datetime.time]) -> str:
 
 
 ENCODERS_BY_TYPE: Dict[Type[Any], Callable[[Any], Any]] = {
+    Color: str,
     IPv4Address: str,
     IPv6Address: str,
     IPv4Interface: str,
     IPv6Interface: str,
     IPv4Network: str,
     IPv6Network: str,
-    SecretStr: lambda ss: ss.display(),
-    SecretBytes: lambda sb: sb.display(),
+    SecretStr: str,
+    SecretBytes: str,
     UUID: str,
     datetime.datetime: isoformat,
     datetime.date: isoformat,
@@ -47,6 +50,8 @@ def pydantic_encoder(obj: Any) -> Any:
         return obj.value
     elif isinstance(obj, Path):
         return str(obj)
+    elif is_dataclass(obj):
+        return asdict(obj)
 
     try:
         encoder = ENCODERS_BY_TYPE[type(obj)]
