@@ -7,61 +7,27 @@ from enum import Enum
 from functools import partial
 from pathlib import Path
 from types import FunctionType
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Dict,
-    Generator,
-    List,
-    Optional,
-    Set,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    cast,
-    no_type_check,
-)
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Type, TypeVar, Union, cast, no_type_check
 
-from .class_validators import ValidatorGroup, extract_validators, inherit_validators
+from .class_validators import ROOT_KEY, ValidatorGroup, extract_root_validators, extract_validators, inherit_validators
 from .error_wrappers import ErrorWrapper, ValidationError
 from .errors import ConfigError, DictError, ExtraError, MissingError
-from .fields import Field, Shape
+from .fields import SHAPE_MAPPING, ModelField, Undefined
 from .json import custom_pydantic_encoder, pydantic_encoder
 from .parse import Protocol, load_file, load_str_bytes
 from .schema import model_schema
 from .types import PyObject, StrBytes
-from .utils import (
-    AnyCallable,
-    AnyType,
-    ForwardRef,
-    GetterDict,
-    ValueItems,
-    change_exception,
-    is_classvar,
-    resolve_annotations,
-    truncate,
-    update_field_forward_refs,
-    validate_field_name,
-)
+from .typing import AnyCallable, AnyType, ForwardRef, is_classvar, resolve_annotations, update_field_forward_refs
+from .utils import GetterDict, Representation, ValueItems, lenient_issubclass, validate_field_name
 
-if TYPE_CHECKING:  # pragma: no cover
-    from .dataclasses import DataclassType  # noqa: F401
-    from .types import CallableGenerator, ModelOrDc
+if TYPE_CHECKING:
     from .class_validators import ValidatorListDict
+    from .types import ModelOrDc
+    from .typing import CallableGenerator, TupleGenerator, DictStrAny, DictAny, SetStr
+    from .typing import AbstractSetIntStr, DictIntStrAny, ReprArgs  # noqa: F401
 
-    AnyGenerator = Generator[Any, None, None]
-    TupleGenerator = Generator[Tuple[str, Any], None, None]
-    DictStrAny = Dict[str, Any]
     ConfigType = Type['BaseConfig']
-    DictAny = Dict[Any, Any]
-    SetStr = Set[str]
-    ListStr = List[str]
     Model = TypeVar('Model', bound='BaseModel')
-    IntStr = Union[int, str]
-    SetIntStr = Set[IntStr]
-    DictIntStrAny = Dict[IntStr, Any]
 
 try:
     import cython  # type: ignore
@@ -73,6 +39,8 @@ else:  # pragma: no cover
     except AttributeError:
         compiled = False
 
+__all__ = 'BaseConfig', 'BaseModel', 'Extra', 'compiled', 'create_model', 'validate_model'
+
 
 class Extra(str, Enum):
     allow = 'allow'
@@ -88,29 +56,39 @@ class BaseConfig:
     validate_all = False
     extra = Extra.ignore
     allow_mutation = True
-    allow_population_by_alias = False
+    allow_population_by_field_name = False
     use_enum_values = False
     fields: Dict[str, Union[str, Dict[str, str]]] = {}
     validate_assignment = False
     error_msg_templates: Dict[str, str] = {}
     arbitrary_types_allowed = False
-    json_encoders: Dict[AnyType, AnyCallable] = {}
     orm_mode: bool = False
+    getter_dict: Type[GetterDict] = GetterDict
     alias_generator: Optional[Callable[[str], str]] = None
     keep_untouched: Tuple[type, ...] = ()
     schema_extra: Dict[str, Any] = {}
+    json_loads: Callable[[str], Any] = json.loads
+    json_dumps: Callable[..., str] = json.dumps
+    json_encoders: Dict[AnyType, AnyCallable] = {}
 
     @classmethod
-    def get_field_schema(cls, name: str) -> Dict[str, str]:
-        field_config = cls.fields.get(name) or {}
-        if isinstance(field_config, str):
-            field_config = {'alias': field_config}
-        elif cls.alias_generator and 'alias' not in field_config:
+    def get_field_info(cls, name: str) -> Dict[str, Any]:
+        field_info = cls.fields.get(name) or {}
+        if isinstance(field_info, str):
+            field_info = {'alias': field_info}
+        elif cls.alias_generator and 'alias' not in field_info:
             alias = cls.alias_generator(name)
             if not isinstance(alias, str):
                 raise TypeError(f'Config.alias_generator must return str, not {type(alias)}')
-            field_config['alias'] = alias
-        return field_config
+            field_info['alias'] = alias
+        return field_info
+
+    @classmethod
+    def prepare_field(cls, field: 'ModelField') -> None:
+        """
+        Optional hook to check or modify fields during model creation.
+        """
+        pass
 
 
 def inherit_config(self_config: 'ConfigType', parent_config: 'ConfigType') -> 'ConfigType':
@@ -123,68 +101,60 @@ def inherit_config(self_config: 'ConfigType', parent_config: 'ConfigType') -> 'C
     return type('Config', base_classes, {})
 
 
-EXTRA_LINK = 'https://pydantic-docs.helpmanual.io/#model-config'
+EXTRA_LINK = 'https://pydantic-docs.helpmanual.io/usage/model_config/'
 
 
-def set_extra(config: Type[BaseConfig], cls_name: str) -> None:
-    has_ignore_extra, has_allow_extra = hasattr(config, 'ignore_extra'), hasattr(config, 'allow_extra')
-    if has_ignore_extra or has_allow_extra:
-        if getattr(config, 'allow_extra', False):
-            config.extra = Extra.allow
-        elif getattr(config, 'ignore_extra', True):
-            config.extra = Extra.ignore
-        else:
-            config.extra = Extra.forbid
-
-        if has_ignore_extra and has_allow_extra:
-            warnings.warn(
-                f'{cls_name}: "ignore_extra" and "allow_extra" are deprecated and replaced by "extra", '
-                f'see {EXTRA_LINK}',
-                DeprecationWarning,
-            )
-        elif has_ignore_extra:
-            warnings.warn(
-                f'{cls_name}: "ignore_extra" is deprecated and replaced by "extra", see {EXTRA_LINK}',
-                DeprecationWarning,
-            )
-        else:
-            warnings.warn(
-                f'{cls_name}: "allow_extra" is deprecated and replaced by "extra", see {EXTRA_LINK}', DeprecationWarning
-            )
-    elif not isinstance(config.extra, Extra):
+def prepare_config(config: Type[BaseConfig], cls_name: str) -> None:
+    if not isinstance(config.extra, Extra):
         try:
             config.extra = Extra(config.extra)
         except ValueError:
             raise ValueError(f'"{cls_name}": {config.extra} is not a valid value for "extra"')
 
+    if hasattr(config, 'allow_population_by_alias'):
+        warnings.warn(
+            f'{cls_name}: "allow_population_by_alias" is deprecated and replaced by "allow_population_by_field_name"',
+            DeprecationWarning,
+        )
+        config.allow_population_by_field_name = config.allow_population_by_alias  # type: ignore
+
+    if hasattr(config, 'case_insensitive') and any('BaseSettings.Config' in c.__qualname__ for c in config.__mro__):
+        warnings.warn(
+            f'{cls_name}: "case_insensitive" is deprecated on BaseSettings config and replaced by '
+            f'"case_sensitive" (default False)',
+            DeprecationWarning,
+        )
+        config.case_sensitive = not config.case_insensitive  # type: ignore
+
 
 def is_valid_field(name: str) -> bool:
     if not name.startswith('_'):
         return True
-    return '__root__' == name
+    return ROOT_KEY == name
 
 
-def validate_custom_root_type(fields: Dict[str, Field]) -> None:
+def validate_custom_root_type(fields: Dict[str, ModelField]) -> None:
     if len(fields) > 1:
         raise ValueError('__root__ cannot be mixed with other fields')
-    if fields['__root__'].shape is Shape.MAPPING:
-        raise TypeError('custom root type cannot allow mapping')
 
 
 UNTOUCHED_TYPES = FunctionType, property, type, classmethod, staticmethod
 
 
-class MetaModel(ABCMeta):
-    @no_type_check
-    def __new__(mcs, name, bases, namespace):
-        fields: Dict[str, Field] = {}
+class ModelMetaclass(ABCMeta):
+    @no_type_check  # noqa C901
+    def __new__(mcs, name, bases, namespace, **kwargs):  # noqa C901
+        fields: Dict[str, ModelField] = {}
         config = BaseConfig
         validators: 'ValidatorListDict' = {}
+        pre_root_validators, post_root_validators = [], []
         for base in reversed(bases):
             if issubclass(base, BaseModel) and base != BaseModel:
                 fields.update(deepcopy(base.__fields__))
                 config = inherit_config(base.__config__, config)
                 validators = inherit_validators(base.__validators__, validators)
+                pre_root_validators += base.__pre_root_validators__
+                post_root_validators += base.__post_root_validators__
 
         config = inherit_config(namespace.get('Config'), config)
         validators = inherit_validators(extract_validators(namespace), validators)
@@ -196,44 +166,58 @@ class MetaModel(ABCMeta):
             if extra_validators:
                 f.class_validators.update(extra_validators)
                 # re-run prepare to add extra validators
-                f.prepare()
+                f.populate_validators()
 
-        set_extra(config, name)
-        annotations = resolve_annotations(namespace.get('__annotations__', {}), namespace.get('__module__', None))
+        prepare_config(config, name)
 
         class_vars = set()
         if (namespace.get('__module__'), namespace.get('__qualname__')) != ('pydantic.main', 'BaseModel'):
+            annotations = resolve_annotations(namespace.get('__annotations__', {}), namespace.get('__module__', None))
+            untouched_types = UNTOUCHED_TYPES + config.keep_untouched
             # annotation only fields need to come first in fields
             for ann_name, ann_type in annotations.items():
                 if is_classvar(ann_type):
                     class_vars.add(ann_name)
-                elif is_valid_field(ann_name) and ann_name not in namespace:
+                elif is_valid_field(ann_name):
                     validate_field_name(bases, ann_name)
-                    fields[ann_name] = Field.infer(
+                    value = namespace.get(ann_name, Undefined)
+                    if (
+                        isinstance(value, untouched_types)
+                        and ann_type != PyObject
+                        and not lenient_issubclass(getattr(ann_type, '__origin__', None), Type)
+                    ):
+                        continue
+                    fields[ann_name] = ModelField.infer(
                         name=ann_name,
-                        value=...,
+                        value=value,
                         annotation=ann_type,
                         class_validators=vg.get_validators(ann_name),
                         config=config,
                     )
 
-            untouched_types = UNTOUCHED_TYPES + config.keep_untouched
             for var_name, value in namespace.items():
                 if (
-                    is_valid_field(var_name)
-                    and (annotations.get(var_name) == PyObject or not isinstance(value, untouched_types))
+                    var_name not in annotations
+                    and is_valid_field(var_name)
+                    and not isinstance(value, untouched_types)
                     and var_name not in class_vars
                 ):
                     validate_field_name(bases, var_name)
-                    fields[var_name] = Field.infer(
+                    inferred = ModelField.infer(
                         name=var_name,
                         value=value,
                         annotation=annotations.get(var_name),
                         class_validators=vg.get_validators(var_name),
                         config=config,
                     )
+                    if var_name in fields and inferred.type_ != fields[var_name].type_:
+                        raise TypeError(
+                            f'The type of {name}.{var_name} differs from the new default value; '
+                            f'if you wish to change the type of this field, please use a type annotation'
+                        )
+                    fields[var_name] = inferred
 
-        _custom_root_type = '__root__' in fields
+        _custom_root_type = ROOT_KEY in fields
         if _custom_root_type:
             validate_custom_root_type(fields)
         vg.check_for_unused()
@@ -241,38 +225,53 @@ class MetaModel(ABCMeta):
             json_encoder = partial(custom_pydantic_encoder, config.json_encoders)
         else:
             json_encoder = pydantic_encoder
+        pre_rv_new, post_rv_new = extract_root_validators(namespace)
         new_namespace = {
             '__config__': config,
             '__fields__': fields,
+            '__field_defaults__': {n: f.default for n, f in fields.items() if not f.required},
             '__validators__': vg.validators,
-            '_schema_cache': {},
-            '_json_encoder': staticmethod(json_encoder),
-            '_custom_root_type': _custom_root_type,
+            '__pre_root_validators__': pre_root_validators + pre_rv_new,
+            '__post_root_validators__': post_root_validators + post_rv_new,
+            '__schema_cache__': {},
+            '__json_encoder__': staticmethod(json_encoder),
+            '__custom_root_type__': _custom_root_type,
             **{n: v for n, v in namespace.items() if n not in fields},
         }
-        return super().__new__(mcs, name, bases, new_namespace)
+        return super().__new__(mcs, name, bases, new_namespace, **kwargs)
 
 
-class BaseModel(metaclass=MetaModel):
-    if TYPE_CHECKING:  # pragma: no cover
+class BaseModel(metaclass=ModelMetaclass):
+    if TYPE_CHECKING:
         # populated by the metaclass, defined here to help IDEs only
-        __fields__: Dict[str, Field] = {}
+        __fields__: Dict[str, ModelField] = {}
+        __field_defaults__: Dict[str, Any] = {}
         __validators__: Dict[str, AnyCallable] = {}
+        __pre_root_validators__: List[AnyCallable]
+        __post_root_validators__: List[AnyCallable]
         __config__: Type[BaseConfig] = BaseConfig
         __root__: Any = None
-        _json_encoder: Callable[[Any], Any] = lambda x: x
-        _schema_cache: 'DictAny' = {}
-        _custom_root_type: bool = False
+        __json_encoder__: Callable[[Any], Any] = lambda x: x
+        __schema_cache__: 'DictAny' = {}
+        __custom_root_type__: bool = False
 
     Config = BaseConfig
     __slots__ = ('__dict__', '__fields_set__')
+    # equivalent of inheriting from Representation
+    __repr_name__ = Representation.__repr_name__
+    __repr_str__ = Representation.__repr_str__
+    __pretty__ = Representation.__pretty__
+    __str__ = Representation.__str__
+    __repr__ = Representation.__repr__
 
     def __init__(__pydantic_self__, **data: Any) -> None:
         # Uses something other than `self` the first arg to allow "self" as a settable attribute
-        if TYPE_CHECKING:  # pragma: no cover
+        if TYPE_CHECKING:
             __pydantic_self__.__dict__: Dict[str, Any] = {}
             __pydantic_self__.__fields_set__: 'SetStr' = set()
-        values, fields_set, _ = validate_model(__pydantic_self__, data)
+        values, fields_set, validation_error = validate_model(__pydantic_self__.__class__, data)
+        if validation_error:
+            raise validation_error
         object.__setattr__(__pydantic_self__, '__dict__', values)
         object.__setattr__(__pydantic_self__, '__fields_set__', fields_set)
 
@@ -283,7 +282,7 @@ class BaseModel(metaclass=MetaModel):
         elif not self.__config__.allow_mutation:
             raise TypeError(f'"{self.__class__.__name__}" is immutable and does not support item assignment')
         elif self.__config__.validate_assignment:
-            known_field = self.fields.get(name, None)
+            known_field = self.__fields__.get(name, None)
             if known_field:
                 value, error_ = known_field.validate(value, self.dict(exclude={name}), loc=name)
                 if error_:
@@ -301,26 +300,38 @@ class BaseModel(metaclass=MetaModel):
     def dict(
         self,
         *,
-        include: Union['SetIntStr', 'DictIntStrAny'] = None,
-        exclude: Union['SetIntStr', 'DictIntStrAny'] = None,
+        include: Union['AbstractSetIntStr', 'DictIntStrAny'] = None,
+        exclude: Union['AbstractSetIntStr', 'DictIntStrAny'] = None,
         by_alias: bool = False,
-        skip_defaults: bool = False,
+        skip_defaults: bool = None,
+        exclude_unset: bool = False,
+        exclude_defaults: bool = False,
+        exclude_none: bool = False,
     ) -> 'DictStrAny':
         """
         Generate a dictionary representation of the model, optionally specifying which fields to include or exclude.
         """
+        if skip_defaults is not None:
+            warnings.warn(
+                f'{self.__class__.__name__}.dict(): "skip_defaults" is deprecated and replaced by "exclude_unset"',
+                DeprecationWarning,
+            )
+            exclude_unset = skip_defaults
         get_key = self._get_key_factory(by_alias)
-        get_key = partial(get_key, self.fields)
+        get_key = partial(get_key, self.__fields__)
 
-        allowed_keys = self._calculate_keys(include=include, exclude=exclude, skip_defaults=skip_defaults)
+        allowed_keys = self._calculate_keys(include=include, exclude=exclude, exclude_unset=exclude_unset)
         return {
             get_key(k): v
             for k, v in self._iter(
+                to_dict=True,
                 by_alias=by_alias,
                 allowed_keys=allowed_keys,
                 include=include,
                 exclude=exclude,
-                skip_defaults=skip_defaults,
+                exclude_unset=exclude_unset,
+                exclude_defaults=exclude_defaults,
+                exclude_none=exclude_none,
             )
         }
 
@@ -333,10 +344,13 @@ class BaseModel(metaclass=MetaModel):
     def json(
         self,
         *,
-        include: Union['SetIntStr', 'DictIntStrAny'] = None,
-        exclude: Union['SetIntStr', 'DictIntStrAny'] = None,
+        include: Union['AbstractSetIntStr', 'DictIntStrAny'] = None,
+        exclude: Union['AbstractSetIntStr', 'DictIntStrAny'] = None,
         by_alias: bool = False,
-        skip_defaults: bool = False,
+        skip_defaults: bool = None,
+        exclude_unset: bool = False,
+        exclude_defaults: bool = False,
+        exclude_none: bool = False,
         encoder: Optional[Callable[[Any], Any]] = None,
         **dumps_kwargs: Any,
     ) -> str:
@@ -345,24 +359,37 @@ class BaseModel(metaclass=MetaModel):
 
         `encoder` is an optional function to supply as `default` to json.dumps(), other arguments as per `json.dumps()`.
         """
-        encoder = cast(Callable[[Any], Any], encoder or self._json_encoder)
-        return json.dumps(
-            self.dict(include=include, exclude=exclude, by_alias=by_alias, skip_defaults=skip_defaults),
-            default=encoder,
-            **dumps_kwargs,
+        if skip_defaults is not None:
+            warnings.warn(
+                f'{self.__class__.__name__}.json(): "skip_defaults" is deprecated and replaced by "exclude_unset"',
+                DeprecationWarning,
+            )
+            exclude_unset = skip_defaults
+        encoder = cast(Callable[[Any], Any], encoder or self.__json_encoder__)
+        data = self.dict(
+            include=include,
+            exclude=exclude,
+            by_alias=by_alias,
+            exclude_unset=exclude_unset,
+            exclude_defaults=exclude_defaults,
+            exclude_none=exclude_none,
         )
+        if self.__custom_root_type__:
+            data = data[ROOT_KEY]
+        return self.__config__.json_dumps(data, default=encoder, **dumps_kwargs)
 
     @classmethod
     def parse_obj(cls: Type['Model'], obj: Any) -> 'Model':
-        if not isinstance(obj, dict):
-            if cls._custom_root_type:
-                obj = {'__root__': obj}
-            else:
-                try:
-                    obj = dict(obj)
-                except (TypeError, ValueError) as e:
-                    exc = TypeError(f'{cls.__name__} expected dict not {type(obj).__name__}')
-                    raise ValidationError([ErrorWrapper(exc, loc='__obj__')], cls) from e
+        if cls.__custom_root_type__ and (
+            not (isinstance(obj, dict) and obj.keys() == {ROOT_KEY}) or cls.__fields__[ROOT_KEY].shape == SHAPE_MAPPING
+        ):
+            obj = {ROOT_KEY: obj}
+        elif not isinstance(obj, dict):
+            try:
+                obj = dict(obj)
+            except (TypeError, ValueError) as e:
+                exc = TypeError(f'{cls.__name__} expected dict not {type(obj).__name__}')
+                raise ValidationError([ErrorWrapper(exc, loc=ROOT_KEY)], cls) from e
         return cls(**obj)
 
     @classmethod
@@ -377,10 +404,15 @@ class BaseModel(metaclass=MetaModel):
     ) -> 'Model':
         try:
             obj = load_str_bytes(
-                b, proto=proto, content_type=content_type, encoding=encoding, allow_pickle=allow_pickle
+                b,
+                proto=proto,
+                content_type=content_type,
+                encoding=encoding,
+                allow_pickle=allow_pickle,
+                json_loads=cls.__config__.json_loads,
             )
         except (ValueError, TypeError, UnicodeDecodeError) as e:
-            raise ValidationError([ErrorWrapper(e, loc='__obj__')], cls)
+            raise ValidationError([ErrorWrapper(e, loc=ROOT_KEY)], cls)
         return cls.parse_obj(obj)
 
     @classmethod
@@ -402,27 +434,31 @@ class BaseModel(metaclass=MetaModel):
             raise ConfigError('You must have the config attribute orm_mode=True to use from_orm')
         obj = cls._decompose_class(obj)
         m = cls.__new__(cls)
-        values, fields_set, _ = validate_model(m, obj)
+        values, fields_set, validation_error = validate_model(cls, obj)
+        if validation_error:
+            raise validation_error
         object.__setattr__(m, '__dict__', values)
         object.__setattr__(m, '__fields_set__', fields_set)
         return m
 
     @classmethod
-    def construct(cls: Type['Model'], values: 'DictAny', fields_set: 'SetStr') -> 'Model':
+    def construct(cls: Type['Model'], _fields_set: Optional['SetStr'] = None, **values: Any) -> 'Model':
         """
-        Creates a new model and set __dict__ without any validation, thus values should already be trusted.
-        Chances are you don't want to use this method directly.
+        Creates a new model setting __dict__ and __fields_set__ from trusted or pre-validated data.
+        Default values are respected, but no other validation is performed.
         """
         m = cls.__new__(cls)
-        object.__setattr__(m, '__dict__', values)
-        object.__setattr__(m, '__fields_set__', fields_set)
+        object.__setattr__(m, '__dict__', {**deepcopy(cls.__field_defaults__), **values})
+        if _fields_set is None:
+            _fields_set = set(values.keys())
+        object.__setattr__(m, '__fields_set__', _fields_set)
         return m
 
     def copy(
         self: 'Model',
         *,
-        include: Union['SetIntStr', 'DictIntStrAny'] = None,
-        exclude: Union['SetIntStr', 'DictIntStrAny'] = None,
+        include: Union['AbstractSetIntStr', 'DictIntStrAny'] = None,
+        exclude: Union['AbstractSetIntStr', 'DictIntStrAny'] = None,
         update: 'DictStrAny' = None,
         deep: bool = False,
     ) -> 'Model':
@@ -440,7 +476,7 @@ class BaseModel(metaclass=MetaModel):
             # skip constructing values if no arguments are passed
             v = self.__dict__
         else:
-            allowed_keys = self._calculate_keys(include=include, exclude=exclude, skip_defaults=False, update=update)
+            allowed_keys = self._calculate_keys(include=include, exclude=exclude, exclude_unset=False, update=update)
             if allowed_keys is None:
                 v = {**self.__dict__, **(update or {})}
             else:
@@ -451,7 +487,7 @@ class BaseModel(metaclass=MetaModel):
                             by_alias=False,
                             include=include,
                             exclude=exclude,
-                            skip_defaults=False,
+                            exclude_unset=False,
                             allowed_keys=allowed_keys,
                         )
                     ),
@@ -460,27 +496,27 @@ class BaseModel(metaclass=MetaModel):
 
         if deep:
             v = deepcopy(v)
-        m = self.__class__.construct(v, self.__fields_set__.copy())
-        return m
 
-    @property
-    def fields(self) -> Dict[str, Field]:
-        return self.__fields__
+        cls = self.__class__
+        m = cls.__new__(cls)
+        object.__setattr__(m, '__dict__', v)
+        object.__setattr__(m, '__fields_set__', self.__fields_set__.copy())
+        return m
 
     @classmethod
     def schema(cls, by_alias: bool = True) -> 'DictStrAny':
-        cached = cls._schema_cache.get(by_alias)
+        cached = cls.__schema_cache__.get(by_alias)
         if cached is not None:
             return cached
         s = model_schema(cls, by_alias=by_alias)
-        cls._schema_cache[by_alias] = s
+        cls.__schema_cache__[by_alias] = s
         return s
 
     @classmethod
     def schema_json(cls, *, by_alias: bool = True, **dumps_kwargs: Any) -> str:
         from .json import pydantic_encoder
 
-        return json.dumps(cls.schema(by_alias=by_alias), default=pydantic_encoder, **dumps_kwargs)
+        return cls.__config__.json_dumps(cls.schema(by_alias=by_alias), default=pydantic_encoder, **dumps_kwargs)
 
     @classmethod
     def __get_validators__(cls) -> 'CallableGenerator':
@@ -495,12 +531,15 @@ class BaseModel(metaclass=MetaModel):
         elif cls.__config__.orm_mode:
             return cls.from_orm(value)
         else:
-            with change_exception(DictError, TypeError, ValueError):
-                return cls(**dict(value))
+            try:
+                value_as_dict = dict(value)
+            except (TypeError, ValueError) as e:
+                raise DictError() from e
+            return cls(**value_as_dict)
 
     @classmethod
     def _decompose_class(cls: Type['Model'], obj: Any) -> GetterDict:
-        return GetterDict(obj)
+        return cls.__config__.getter_dict(obj)
 
     @classmethod
     @no_type_check
@@ -509,14 +548,23 @@ class BaseModel(metaclass=MetaModel):
         v: Any,
         to_dict: bool,
         by_alias: bool,
-        include: Optional[Union['SetIntStr', 'DictIntStrAny']],
-        exclude: Optional[Union['SetIntStr', 'DictIntStrAny']],
-        skip_defaults: bool,
+        include: Optional[Union['AbstractSetIntStr', 'DictIntStrAny']],
+        exclude: Optional[Union['AbstractSetIntStr', 'DictIntStrAny']],
+        exclude_unset: bool,
+        exclude_defaults: bool,
+        exclude_none: bool,
     ) -> Any:
 
         if isinstance(v, BaseModel):
             if to_dict:
-                return v.dict(by_alias=by_alias, skip_defaults=skip_defaults, include=include, exclude=exclude)
+                return v.dict(
+                    by_alias=by_alias,
+                    exclude_unset=exclude_unset,
+                    exclude_defaults=exclude_defaults,
+                    include=include,
+                    exclude=exclude,
+                    exclude_none=exclude_none,
+                )
             else:
                 return v.copy(include=include, exclude=exclude)
 
@@ -529,9 +577,11 @@ class BaseModel(metaclass=MetaModel):
                     v_,
                     to_dict=to_dict,
                     by_alias=by_alias,
-                    skip_defaults=skip_defaults,
+                    exclude_unset=exclude_unset,
+                    exclude_defaults=exclude_defaults,
                     include=value_include and value_include.for_element(k_),
                     exclude=value_exclude and value_exclude.for_element(k_),
+                    exclude_none=exclude_none,
                 )
                 for k_, v_ in v.items()
                 if (not value_exclude or not value_exclude.is_excluded(k_))
@@ -544,9 +594,11 @@ class BaseModel(metaclass=MetaModel):
                     v_,
                     to_dict=to_dict,
                     by_alias=by_alias,
-                    skip_defaults=skip_defaults,
+                    exclude_unset=exclude_unset,
+                    exclude_defaults=exclude_defaults,
                     include=value_include and value_include.for_element(i),
                     exclude=value_exclude and value_exclude.for_element(i),
+                    exclude_none=exclude_none,
                 )
                 for i, v_ in enumerate(v)
                 if (not value_exclude or not value_exclude.is_excluded(i))
@@ -566,7 +618,7 @@ class BaseModel(metaclass=MetaModel):
         for f in cls.__fields__.values():
             update_field_forward_refs(f, globalns=globalns, localns=localns)
 
-    def __iter__(self) -> 'AnyGenerator':
+    def __iter__(self) -> 'TupleGenerator':
         """
         so `dict(model)` works
         """
@@ -574,39 +626,52 @@ class BaseModel(metaclass=MetaModel):
 
     def _iter(
         self,
-        to_dict: bool = True,
+        to_dict: bool = False,
         by_alias: bool = False,
         allowed_keys: Optional['SetStr'] = None,
-        include: Union['SetIntStr', 'DictIntStrAny'] = None,
-        exclude: Union['SetIntStr', 'DictIntStrAny'] = None,
-        skip_defaults: bool = False,
+        include: Union['AbstractSetIntStr', 'DictIntStrAny'] = None,
+        exclude: Union['AbstractSetIntStr', 'DictIntStrAny'] = None,
+        exclude_unset: bool = False,
+        exclude_defaults: bool = False,
+        exclude_none: bool = False,
     ) -> 'TupleGenerator':
 
         value_exclude = ValueItems(self, exclude) if exclude else None
         value_include = ValueItems(self, include) if include else None
 
+        if exclude_defaults:
+            if allowed_keys is None:
+                allowed_keys = set(self.__fields__)
+            for k, v in self.__field_defaults__.items():
+                if self.__dict__[k] == v:
+                    allowed_keys.discard(k)
+
         for k, v in self.__dict__.items():
             if allowed_keys is None or k in allowed_keys:
-                yield k, self._get_value(
+                value = self._get_value(
                     v,
                     to_dict=to_dict,
                     by_alias=by_alias,
                     include=value_include and value_include.for_element(k),
                     exclude=value_exclude and value_exclude.for_element(k),
-                    skip_defaults=skip_defaults,
+                    exclude_unset=exclude_unset,
+                    exclude_defaults=exclude_defaults,
+                    exclude_none=exclude_none,
                 )
+                if not (exclude_none and value is None):
+                    yield k, value
 
     def _calculate_keys(
         self,
-        include: Optional[Union['SetIntStr', 'DictIntStrAny']],
-        exclude: Optional[Union['SetIntStr', 'DictIntStrAny']],
-        skip_defaults: bool,
+        include: Optional[Union['AbstractSetIntStr', 'DictIntStrAny']],
+        exclude: Optional[Union['AbstractSetIntStr', 'DictIntStrAny']],
+        exclude_unset: bool,
         update: Optional['DictStrAny'] = None,
     ) -> Optional['SetStr']:
-        if include is None and exclude is None and skip_defaults is False:
+        if include is None and exclude is None and exclude_unset is False:
             return None
 
-        if skip_defaults:
+        if exclude_unset:
             keys = self.__fields_set__.copy()
         else:
             keys = set(self.__dict__.keys())
@@ -634,19 +699,17 @@ class BaseModel(metaclass=MetaModel):
         else:
             return self.dict() == other
 
-    def __repr__(self) -> str:
-        return f'<{self}>'
+    def __repr_args__(self) -> 'ReprArgs':
+        return self.__dict__.items()  # type: ignore
+
+    @property
+    def fields(self) -> Dict[str, ModelField]:
+        warnings.warn('`fields` attribute is deprecated, use `__fields__` instead', DeprecationWarning)
+        return self.__fields__
 
     def to_string(self, pretty: bool = False) -> str:
-        divider = '\n  ' if pretty else ' '
-        return '{}{}{}'.format(
-            self.__class__.__name__,
-            divider,
-            divider.join('{}={}'.format(k, truncate(v)) for k, v in self.__dict__.items()),
-        )
-
-    def __str__(self) -> str:
-        return self.to_string()
+        warnings.warn('`model.to_string()` method is deprecated, use `str(model)` instead', DeprecationWarning)
+        return str(self)
 
     @property
     def __values__(self) -> 'DictStrAny':
@@ -714,7 +777,7 @@ _missing = object()
 
 
 def validate_model(  # noqa: C901 (ignore complexity)
-    model: Union[BaseModel, Type[BaseModel]], input_data: 'DictStrAny', raise_exc: bool = True, cls: 'ModelOrDc' = None
+    model: Type[BaseModel], input_data: 'DictStrAny', cls: 'ModelOrDc' = None
 ) -> Tuple['DictStrAny', 'SetStr', Optional[ValidationError]]:
     """
     validate data against a model.
@@ -727,26 +790,39 @@ def validate_model(  # noqa: C901 (ignore complexity)
     fields_set = set()
     config = model.__config__
     check_extra = config.extra is not Extra.ignore
+    cls_ = cls or model
+
+    for validator in model.__pre_root_validators__:
+        try:
+            input_data = validator(cls_, input_data)
+        except (ValueError, TypeError, AssertionError) as exc:
+            return {}, set(), ValidationError([ErrorWrapper(exc, loc=ROOT_KEY)], cls_)
 
     for name, field in model.__fields__.items():
         if type(field.type_) == ForwardRef:
             raise ConfigError(
                 f'field "{field.name}" not yet prepared so type is still a ForwardRef, '
-                f'you might need to call {model.__class__.__name__}.update_forward_refs().'
+                f'you might need to call {cls_.__name__}.update_forward_refs().'
             )
 
         value = input_data.get(field.alias, _missing)
         using_name = False
-        if value is _missing and config.allow_population_by_alias and field.alt_alias:
+        if value is _missing and config.allow_population_by_field_name and field.alt_alias:
             value = input_data.get(field.name, _missing)
             using_name = True
 
         if value is _missing:
             if field.required:
-                errors.append(ErrorWrapper(MissingError(), loc=field.alias, config=model.__config__))
+                errors.append(ErrorWrapper(MissingError(), loc=field.alias))
                 continue
-            value = deepcopy(field.default)
-            if not model.__config__.validate_all and not field.validate_always:
+
+            if field.default is None:
+                # deepcopy is quite slow on None
+                value = None
+            else:
+                value = deepcopy(field.default)
+
+            if not config.validate_all and not field.validate_always:
                 values[name] = value
                 continue
         else:
@@ -754,7 +830,7 @@ def validate_model(  # noqa: C901 (ignore complexity)
             if check_extra:
                 names_used.add(field.name if using_name else field.alias)
 
-        v_, errors_ = field.validate(value, values, loc=field.alias, cls=cls or model.__class__)  # type: ignore
+        v_, errors_ = field.validate(value, values, loc=field.alias, cls=cls_)
         if isinstance(errors_, ErrorWrapper):
             errors.append(errors_)
         elif isinstance(errors_, list):
@@ -763,7 +839,10 @@ def validate_model(  # noqa: C901 (ignore complexity)
             values[name] = v_
 
     if check_extra:
-        extra = input_data.keys() - names_used
+        if isinstance(input_data, GetterDict):
+            extra = input_data.extra_keys() - names_used
+        else:
+            extra = input_data.keys() - names_used
         if extra:
             fields_set |= extra
             if config.extra is Extra.allow:
@@ -771,17 +850,16 @@ def validate_model(  # noqa: C901 (ignore complexity)
                     values[f] = input_data[f]
             else:
                 for f in sorted(extra):
-                    errors.append(ErrorWrapper(ExtraError(), loc=f, config=config))
+                    errors.append(ErrorWrapper(ExtraError(), loc=f))
+
+    for validator in model.__post_root_validators__:
+        try:
+            values = validator(cls_, values)
+        except (ValueError, TypeError, AssertionError) as exc:
+            errors.append(ErrorWrapper(exc, loc=ROOT_KEY))
+            break
 
-    err = None
     if errors:
-        model_type = model if isinstance(model, type) else type(model)
-        err = ValidationError(errors, model_type)
-
-    if not raise_exc:
-        return values, fields_set, err
-
-    if err:
-        raise err
-
-    return values, fields_set, None
+        return values, fields_set, ValidationError(errors, cls_)
+    else:
+        return values, fields_set, None
