@@ -4,17 +4,43 @@ import warnings
 from datetime import date, datetime, time, timedelta
 from decimal import Decimal
 from enum import Enum
+from ipaddress import IPv4Address, IPv4Interface, IPv4Network, IPv6Address, IPv6Interface, IPv6Network
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, Type, TypeVar, Union, cast
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    FrozenSet,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 from uuid import UUID
 
-import pydantic
-from pydantic.color import Color
-
-from .fields import Field, Shape
+from .class_validators import ROOT_KEY
+from .color import Color
+from .fields import (
+    SHAPE_FROZENSET,
+    SHAPE_LIST,
+    SHAPE_MAPPING,
+    SHAPE_SEQUENCE,
+    SHAPE_SET,
+    SHAPE_SINGLETON,
+    SHAPE_TUPLE,
+    SHAPE_TUPLE_ELLIPSIS,
+    FieldInfo,
+    ModelField,
+)
 from .json import pydantic_encoder
+from .networks import AnyUrl, EmailStr, IPvAnyAddress, IPvAnyInterface, IPvAnyNetwork, NameEmail
 from .types import (
-    DSN,
     UUID1,
     UUID3,
     UUID4,
@@ -25,158 +51,35 @@ from .types import (
     ConstrainedList,
     ConstrainedStr,
     DirectoryPath,
-    EmailStr,
     FilePath,
-    IPv4Address,
-    IPv4Interface,
-    IPv4Network,
-    IPv6Address,
-    IPv6Interface,
-    IPv6Network,
-    IPvAnyAddress,
-    IPvAnyInterface,
-    IPvAnyNetwork,
     Json,
-    NameEmail,
     SecretBytes,
     SecretStr,
     StrictBool,
-    UrlStr,
+    conbytes,
     condecimal,
     confloat,
     conint,
     conlist,
     constr,
 )
-from .utils import (
+from .typing import (
+    ForwardRef,
+    Literal,
     is_callable_type,
     is_literal_type,
     is_new_type,
-    lenient_issubclass,
     literal_values,
     new_type_supertype,
 )
+from .utils import lenient_issubclass
 
-if TYPE_CHECKING:  # pragma: no cover
-    from . import dataclasses  # noqa: F401
-
-    BaseModel = pydantic.main.BaseModel
-
-
-__all__ = [
-    'Schema',
-    'schema',
-    'model_schema',
-    'field_schema',
-    'get_model_name_map',
-    'get_flat_models_from_model',
-    'get_flat_models_from_field',
-    'get_flat_models_from_fields',
-    'get_flat_models_from_models',
-    'get_long_model_name',
-    'field_type_schema',
-    'model_process_schema',
-    'model_type_schema',
-    'field_singleton_sub_fields_schema',
-    'field_singleton_schema',
-    'get_annotation_from_schema',
-]
+if TYPE_CHECKING:
+    from .main import BaseModel  # noqa: F401
 
 default_prefix = '#/definitions/'
 
 
-class Schema:
-    """
-    Used to provide extra information about a field in a model schema. The parameters will be
-    converted to validations and will add annotations to the generated JSON Schema. Some arguments
-    apply only to number fields (``int``, ``float``, ``Decimal``) and some apply only to ``str``
-
-    :param default: since the Schema is replacing the field’s default, its first argument is used
-      to set the default, use ellipsis (``...``) to indicate the field is required
-    :param alias: the public name of the field
-    :param title: can be any string, used in the schema
-    :param description: can be any string, used in the schema
-    :param const: this field is required and *must* take it's default value
-    :param gt: only applies to numbers, requires the field to be "greater than". The schema
-      will have an ``exclusiveMinimum`` validation keyword
-    :param ge: only applies to numbers, requires the field to be "greater than or equal to". The
-      schema will have a ``minimum`` validation keyword
-    :param lt: only applies to numbers, requires the field to be "less than". The schema
-      will have an ``exclusiveMaximum`` validation keyword
-    :param le: only applies to numbers, requires the field to be "less than or equal to". The
-      schema will have a ``maximum`` validation keyword
-    :param multiple_of: only applies to numbers, requires the field to be "a multiple of". The
-      schema will have a ``multipleOf`` validation keyword
-    :param min_length: only applies to strings, requires the field to have a minimum length. The
-      schema will have a ``maximum`` validation keyword
-    :param max_length: only applies to strings, requires the field to have a maximum length. The
-      schema will have a ``maxLength`` validation keyword
-    :param regex: only applies to strings, requires the field match agains a regular expression
-      pattern string. The schema will have a ``pattern`` validation keyword
-    :param **extra: any additional keyword arguments will be added as is to the schema
-    """
-
-    __slots__ = (
-        'default',
-        'alias',
-        'title',
-        'description',
-        'const',
-        'gt',
-        'ge',
-        'lt',
-        'le',
-        'multiple_of',
-        'min_items',
-        'max_items',
-        'min_length',
-        'max_length',
-        'regex',
-        'extra',
-    )
-
-    def __init__(
-        self,
-        default: Any,
-        *,
-        alias: str = None,
-        title: str = None,
-        description: str = None,
-        const: bool = None,
-        gt: float = None,
-        ge: float = None,
-        lt: float = None,
-        le: float = None,
-        multiple_of: float = None,
-        min_items: int = None,
-        max_items: int = None,
-        min_length: int = None,
-        max_length: int = None,
-        regex: str = None,
-        **extra: Any,
-    ) -> None:
-        self.default = default
-        self.alias = alias
-        self.title = title
-        self.description = description
-        self.const = const
-        self.extra = extra
-        self.gt = gt
-        self.ge = ge
-        self.lt = lt
-        self.le = le
-        self.multiple_of = multiple_of
-        self.min_items = min_items
-        self.max_items = max_items
-        self.min_length = min_length
-        self.max_length = max_length
-        self.regex = regex
-
-    def __repr__(self) -> str:
-        attrs = ((s, getattr(self, s)) for s in self.__slots__)
-        return 'Schema({})'.format(', '.join(f'{a}: {v!r}' for a, v in attrs if v is not None))
-
-
 def schema(
     models: Sequence[Type['BaseModel']],
     *,
@@ -253,7 +156,7 @@ def model_schema(model: Type['BaseModel'], by_alias: bool = True, ref_prefix: Op
 
 
 def field_schema(
-    field: Field,
+    field: ModelField,
     *,
     by_alias: bool = True,
     model_name_map: Dict[Type['BaseModel'], str],
@@ -266,7 +169,7 @@ def field_schema(
     is a model and has sub-models, and those sub-models don't have overrides (as ``title``, ``default``, etc), they
     will be included in the definitions and referenced in the schema instead of included recursively.
 
-    :param field: a Pydantic ``Field``
+    :param field: a Pydantic ``ModelField``
     :param by_alias: use the defined alias (if any) in the returned schema
     :param model_name_map: used to generate the JSON Schema references to other models included in the definitions
     :param ref_prefix: the JSON Pointer prefix to use for references to other schemas, if None, the default of
@@ -276,16 +179,15 @@ def field_schema(
     """
     ref_prefix = ref_prefix or default_prefix
     schema_overrides = False
-    schema = cast('Schema', field.schema)
-    s = dict(title=schema.title or field.alias.title())
-    if schema.title:
+    s = dict(title=field.field_info.title or field.alias.title().replace('_', ' '))
+    if field.field_info.title:
         schema_overrides = True
 
-    if schema.description:
-        s['description'] = schema.description
+    if field.field_info.description:
+        s['description'] = field.field_info.description
         schema_overrides = True
 
-    if not field.required and not (field.schema is not None and field.schema.const) and field.default is not None:
+    if not field.required and not field.field_info.const and field.default is not None:
         s['default'] = encode_default(field.default)
         schema_overrides = True
 
@@ -326,27 +228,26 @@ _numeric_types_attrs: Tuple[Tuple[str, Union[type, Tuple[type, ...]], str], ...]
 )
 
 
-def get_field_schema_validations(field: Field) -> Dict[str, Any]:
+def get_field_schema_validations(field: ModelField) -> Dict[str, Any]:
     """
     Get the JSON Schema validation keywords for a ``field`` with an annotation of
-    a Pydantic ``Schema`` with validation arguments.
+    a Pydantic ``FieldInfo`` with validation arguments.
     """
     f_schema: Dict[str, Any] = {}
     if lenient_issubclass(field.type_, (str, bytes)):
         for attr_name, t, keyword in _str_types_attrs:
-            attr = getattr(field.schema, attr_name, None)
+            attr = getattr(field.field_info, attr_name, None)
             if isinstance(attr, t):
                 f_schema[keyword] = attr
     if lenient_issubclass(field.type_, numeric_types) and not issubclass(field.type_, bool):
         for attr_name, t, keyword in _numeric_types_attrs:
-            attr = getattr(field.schema, attr_name, None)
+            attr = getattr(field.field_info, attr_name, None)
             if isinstance(attr, t):
                 f_schema[keyword] = attr
-    if field.schema is not None and field.schema.const:
+    if field.field_info is not None and field.field_info.const:
         f_schema['const'] = field.default
-    schema = cast('Schema', field.schema)
-    if schema.extra:
-        f_schema.update(schema.extra)
+    if field.field_info.extra:
+        f_schema.update(field.field_info.extra)
     return f_schema
 
 
@@ -395,46 +296,48 @@ def get_flat_models_from_model(
     flat_models: Set[Type['BaseModel']] = set()
     flat_models.add(model)
     known_models |= flat_models
-    fields = cast(Sequence[Field], model.__fields__.values())
+    fields = cast(Sequence[ModelField], model.__fields__.values())
     flat_models |= get_flat_models_from_fields(fields, known_models=known_models)
     return flat_models
 
 
-def get_flat_models_from_field(field: Field, known_models: Set[Type['BaseModel']]) -> Set[Type['BaseModel']]:
+def get_flat_models_from_field(field: ModelField, known_models: Set[Type['BaseModel']]) -> Set[Type['BaseModel']]:
     """
-    Take a single Pydantic ``Field`` (from a model) that could have been declared as a sublcass of BaseModel
+    Take a single Pydantic ``ModelField`` (from a model) that could have been declared as a sublcass of BaseModel
     (so, it could be a submodel), and generate a set with its model and all the sub-models in the tree.
     I.e. if you pass a field that was declared to be of type ``Foo`` (subclass of BaseModel) as ``field``, and that
     model ``Foo`` has a field of type ``Bar`` (also subclass of ``BaseModel``) and that model ``Bar`` has a field of
     type ``Baz`` (also subclass of ``BaseModel``), the return value will be ``set([Foo, Bar, Baz])``.
 
-    :param field: a Pydantic ``Field``
+    :param field: a Pydantic ``ModelField``
     :param known_models: used to solve circular references
     :return: a set with the model used in the declaration for this field, if any, and all its sub-models
     """
-    flat_models: Set[Type['BaseModel']] = set()
+    from .main import BaseModel  # noqa: F811
+
+    flat_models: Set[Type[BaseModel]] = set()
     # Handle dataclass-based models
     field_type = field.type_
-    if lenient_issubclass(getattr(field_type, '__pydantic_model__', None), pydantic.BaseModel):
-        field_type = field_type.__pydantic_model__  # type: ignore
+    if lenient_issubclass(getattr(field_type, '__pydantic_model__', None), BaseModel):
+        field_type = field_type.__pydantic_model__
     if field.sub_fields:
         flat_models |= get_flat_models_from_fields(field.sub_fields, known_models=known_models)
-    elif lenient_issubclass(field_type, pydantic.BaseModel) and field_type not in known_models:
+    elif lenient_issubclass(field_type, BaseModel) and field_type not in known_models:
         flat_models |= get_flat_models_from_model(field_type, known_models=known_models)
     return flat_models
 
 
 def get_flat_models_from_fields(
-    fields: Sequence[Field], known_models: Set[Type['BaseModel']]
+    fields: Sequence[ModelField], known_models: Set[Type['BaseModel']]
 ) -> Set[Type['BaseModel']]:
     """
-    Take a list of Pydantic  ``Field``s (from a model) that could have been declared as sublcasses of ``BaseModel``
+    Take a list of Pydantic  ``ModelField``s (from a model) that could have been declared as sublcasses of ``BaseModel``
     (so, any of them could be a submodel), and generate a set with their models and all the sub-models in the tree.
     I.e. if you pass a the fields of a model ``Foo`` (subclass of ``BaseModel``) as ``fields``, and on of them has a
     field of type ``Bar`` (also subclass of ``BaseModel``) and that model ``Bar`` has a field of type ``Baz`` (also
     subclass of ``BaseModel``), the return value will be ``set([Foo, Bar, Baz])``.
 
-    :param fields: a list of Pydantic ``Field``s
+    :param fields: a list of Pydantic ``ModelField``s
     :param known_models: used to solve circular references
     :return: a set with any model declared in the fields, and all their sub-models
     """
@@ -461,7 +364,7 @@ def get_long_model_name(model: Type['BaseModel']) -> str:
 
 
 def field_type_schema(
-    field: Field,
+    field: ModelField,
     *,
     by_alias: bool,
     model_name_map: Dict[Type['BaseModel'], str],
@@ -478,23 +381,23 @@ def field_type_schema(
     definitions = {}
     nested_models: Set[str] = set()
     ref_prefix = ref_prefix or default_prefix
-    if field.shape is Shape.LIST:
+    if field.shape in {SHAPE_LIST, SHAPE_TUPLE_ELLIPSIS, SHAPE_SEQUENCE, SHAPE_SET, SHAPE_FROZENSET}:
         f_schema, f_definitions, f_nested_models = field_singleton_schema(
             field, by_alias=by_alias, model_name_map=model_name_map, ref_prefix=ref_prefix, known_models=known_models
         )
         definitions.update(f_definitions)
         nested_models.update(f_nested_models)
-        return {'type': 'array', 'items': f_schema}, definitions, nested_models
-    elif field.shape is Shape.SET:
-        f_schema, f_definitions, f_nested_models = field_singleton_schema(
-            field, by_alias=by_alias, model_name_map=model_name_map, ref_prefix=ref_prefix, known_models=known_models
-        )
-        definitions.update(f_definitions)
-        nested_models.update(f_nested_models)
-        return {'type': 'array', 'uniqueItems': True, 'items': f_schema}, definitions, nested_models
-    elif field.shape is Shape.MAPPING:
+        s: Dict[str, Any] = {'type': 'array', 'items': f_schema}
+        if field.shape in {SHAPE_SET, SHAPE_FROZENSET}:
+            s['uniqueItems'] = True
+        if field.field_info.min_items is not None:
+            s['minItems'] = field.field_info.min_items
+        if field.field_info.max_items is not None:
+            s['maxItems'] = field.field_info.max_items
+        return s, definitions, nested_models
+    elif field.shape == SHAPE_MAPPING:
         dict_schema: Dict[str, Any] = {'type': 'object'}
-        key_field = cast(Field, field.key_field)
+        key_field = cast(ModelField, field.key_field)
         regex = getattr(key_field.type_, 'regex', None)
         f_schema, f_definitions, f_nested_models = field_singleton_schema(
             field, by_alias=by_alias, model_name_map=model_name_map, ref_prefix=ref_prefix, known_models=known_models
@@ -509,9 +412,9 @@ def field_type_schema(
             # The dict values are not simply Any, so they need a schema
             dict_schema['additionalProperties'] = f_schema
         return dict_schema, definitions, nested_models
-    elif field.shape is Shape.TUPLE:
+    elif field.shape == SHAPE_TUPLE:
         sub_schema = []
-        sub_fields = cast(List[Field], field.sub_fields)
+        sub_fields = cast(List[ModelField], field.sub_fields)
         for sf in sub_fields:
             sf_schema, sf_definitions, sf_nested_models = field_type_schema(
                 sf, by_alias=by_alias, model_name_map=model_name_map, ref_prefix=ref_prefix, known_models=known_models
@@ -523,7 +426,7 @@ def field_type_schema(
             sub_schema = sub_schema[0]  # type: ignore
         return {'type': 'array', 'items': sub_schema}, definitions, nested_models
     else:
-        assert field.shape is Shape.SINGLETON, field.shape
+        assert field.shape == SHAPE_SINGLETON, field.shape
         f_schema, f_definitions, f_nested_models = field_singleton_schema(
             field,
             by_alias=by_alias,
@@ -604,18 +507,20 @@ def model_type_schema(
             properties[k] = f_schema
             if f.required:
                 required.append(k)
-    if '__root__' in properties:
-        out_schema = properties['__root__']
+    if ROOT_KEY in properties:
+        out_schema = properties[ROOT_KEY]
         out_schema['title'] = model.__config__.title or model.__name__
     else:
         out_schema = {'type': 'object', 'properties': properties}
         if required:
             out_schema['required'] = required
+    if model.__config__.extra == 'forbid':
+        out_schema['additionalProperties'] = False
     return out_schema, definitions, nested_models
 
 
 def field_singleton_sub_fields_schema(
-    sub_fields: Sequence[Field],
+    sub_fields: Sequence[ModelField],
     *,
     by_alias: bool,
     model_name_map: Dict[Type['BaseModel'], str],
@@ -626,7 +531,7 @@ def field_singleton_sub_fields_schema(
     """
     This function is indirectly used by ``field_schema()``, you probably should be using that function.
 
-    Take a list of Pydantic ``Field`` from the declaration of a type with parameters, and generate their
+    Take a list of Pydantic ``ModelField`` from the declaration of a type with parameters, and generate their
     schema. I.e., fields used as "type parameters", like ``str`` and ``int`` in ``Tuple[str, int]``.
     """
     ref_prefix = ref_prefix or default_prefix
@@ -673,8 +578,7 @@ validation_attribute_to_schema_keyword = {
 # Order is important, subclasses of str must go before str, etc
 field_class_to_schema_enum_enabled: Tuple[Tuple[Any, Dict[str, Any]], ...] = (
     (EmailStr, {'type': 'string', 'format': 'email'}),
-    (UrlStr, {'type': 'string', 'format': 'uri'}),
-    (DSN, {'type': 'string', 'format': 'dsn'}),
+    (AnyUrl, {'type': 'string', 'format': 'uri'}),
     (SecretStr, {'type': 'string', 'writeOnly': True}),
     (str, {'type': 'string'}),
     (SecretBytes, {'type': 'string', 'writeOnly': True}),
@@ -697,6 +601,7 @@ field_class_to_schema_enum_enabled: Tuple[Tuple[Any, Dict[str, Any]], ...] = (
     (Color, {'type': 'string', 'format': 'color'}),
 )
 
+json_scheme = {'type': 'string', 'format': 'json-string'}
 
 # Order is important, subclasses of Path must go before Path, etc
 field_class_to_schema_enum_disabled = (
@@ -707,7 +612,7 @@ field_class_to_schema_enum_disabled = (
     (date, {'type': 'string', 'format': 'date'}),
     (time, {'type': 'string', 'format': 'time'}),
     (timedelta, {'type': 'number', 'format': 'time-delta'}),
-    (Json, {'type': 'string', 'format': 'json-string'}),
+    (Json, json_scheme),
     (IPv4Network, {'type': 'string', 'format': 'ipv4network'}),
     (IPv6Network, {'type': 'string', 'format': 'ipv6network'}),
     (IPvAnyNetwork, {'type': 'string', 'format': 'ipvanynetwork'}),
@@ -721,7 +626,7 @@ field_class_to_schema_enum_disabled = (
 
 
 def field_singleton_schema(  # noqa: C901 (ignore complexity)
-    field: Field,
+    field: ModelField,
     *,
     by_alias: bool,
     model_name_map: Dict[Type['BaseModel'], str],
@@ -732,8 +637,9 @@ def field_singleton_schema(  # noqa: C901 (ignore complexity)
     """
     This function is indirectly used by ``field_schema()``, you should probably be using that function.
 
-    Take a single Pydantic ``Field``, and return its schema and any additional definitions from sub-models.
+    Take a single Pydantic ``ModelField``, and return its schema and any additional definitions from sub-models.
     """
+    from .main import BaseModel  # noqa: F811
 
     ref_prefix = ref_prefix or default_prefix
     definitions: Dict[str, Any] = {}
@@ -748,18 +654,29 @@ def field_singleton_schema(  # noqa: C901 (ignore complexity)
             known_models=known_models,
         )
     if field.type_ is Any or type(field.type_) == TypeVar:
-        return {}, definitions, nested_models  # no restrictions
+        if field.parse_json:
+            return json_scheme, definitions, nested_models
+        else:
+            return {}, definitions, nested_models  # no restrictions
     if is_callable_type(field.type_):
         raise SkipField(f'Callable {field.name} was excluded from schema since JSON schema has no equivalent type.')
     f_schema: Dict[str, Any] = {}
-    if field.schema is not None and field.schema.const:
+    if field.field_info is not None and field.field_info.const:
         f_schema['const'] = field.default
     field_type = field.type_
     if is_new_type(field_type):
         field_type = new_type_supertype(field_type)
     if is_literal_type(field_type):
-        # If there were multiple literal values, field.sub_fields would not be falsy
-        literal_value = literal_values(field_type)[0]
+        values = literal_values(field_type)
+        if len(values) > 1:
+            return field_schema(
+                multivalue_literal_field_for_schema(values, field),
+                by_alias=by_alias,
+                model_name_map=model_name_map,
+                ref_prefix=ref_prefix,
+                known_models=known_models,
+            )
+        literal_value = values[0]
         field_type = type(literal_value)
         f_schema['const'] = literal_value
     if issubclass(field_type, Enum):
@@ -782,9 +699,9 @@ def field_singleton_schema(  # noqa: C901 (ignore complexity)
         if issubclass(field_type, type_):
             return t_schema, definitions, nested_models
     # Handle dataclass-based models
-    if lenient_issubclass(getattr(field_type, '__pydantic_model__', None), pydantic.BaseModel):
-        field_type = field_type.__pydantic_model__  # type: ignore
-    if issubclass(field_type, pydantic.BaseModel):
+    if lenient_issubclass(getattr(field_type, '__pydantic_model__', None), BaseModel):
+        field_type = field_type.__pydantic_model__
+    if issubclass(field_type, BaseModel):
         model_name = model_name_map[field_type]
         if field_type not in known_models:
             sub_schema, sub_definitions, sub_nested_models = model_process_schema(
@@ -807,6 +724,19 @@ def field_singleton_schema(  # noqa: C901 (ignore complexity)
     raise ValueError(f'Value not declarable with JSON Schema, field: {field}')
 
 
+def multivalue_literal_field_for_schema(values: Tuple[Any, ...], field: ModelField) -> ModelField:
+    return ModelField(
+        name=field.name,
+        type_=Union[tuple(Literal[value] for value in values)],
+        class_validators=field.class_validators,
+        model_config=field.model_config,
+        default=field.default,
+        required=field.required,
+        alias=field.alias,
+        field_info=field.field_info,
+    )
+
+
 def encode_default(dft: Any) -> Any:
     if isinstance(dft, (int, float, str)):
         return dft
@@ -820,42 +750,98 @@ def encode_default(dft: Any) -> Any:
 
 
 _map_types_constraint: Dict[Any, Callable[..., type]] = {int: conint, float: confloat, Decimal: condecimal}
+_field_constraints = {
+    'min_length',
+    'max_length',
+    'regex',
+    'gt',
+    'lt',
+    'ge',
+    'le',
+    'multiple_of',
+    'min_items',
+    'max_items',
+}
 
 
-def get_annotation_from_schema(annotation: Any, schema: Schema) -> Type[Any]:
+def get_annotation_from_field_info(annotation: Any, field_info: FieldInfo, field_name: str) -> Type[Any]:  # noqa: C901
     """
-    Get an annotation with validation implemented for numbers and strings based on the schema.
+    Get an annotation with validation implemented for numbers and strings based on the field_info.
 
     :param annotation: an annotation from a field specification, as ``str``, ``ConstrainedStr``
-    :param schema: an instance of Schema, possibly with declarations for validations and JSON Schema
+    :param field_info: an instance of FieldInfo, possibly with declarations for validations and JSON Schema
+    :param field_name: name of the field for use in error messages
     :return: the same ``annotation`` if unmodified or a new annotation with validation in place
     """
-    if isinstance(annotation, type):
+    constraints = {f for f in _field_constraints if getattr(field_info, f) is not None}
+    if not constraints:
+        return annotation
+    used_constraints: Set[str] = set()
+
+    def go(type_: Any) -> Type[Any]:
+        if is_literal_type(annotation) or isinstance(type_, ForwardRef) or lenient_issubclass(type_, ConstrainedList):
+            return type_
+        origin = getattr(type_, '__origin__', None)
+        if origin is not None:
+            args: Tuple[Any, ...] = type_.__args__
+            if any(isinstance(a, ForwardRef) for a in args):
+                # forward refs cause infinite recursion below
+                return type_
+
+            if origin is Union:
+                return Union[tuple(go(a) for a in args)]
+
+            if issubclass(origin, List) and (field_info.min_items is not None or field_info.max_items is not None):
+                used_constraints.update({'min_items', 'max_items'})
+                return conlist(go(args[0]), min_items=field_info.min_items, max_items=field_info.max_items)
+
+            for t in (Tuple, List, Set, FrozenSet, Sequence):
+                if issubclass(origin, t):  # type: ignore
+                    return t[tuple(go(a) for a in args)]  # type: ignore
+
+            if issubclass(origin, Dict):
+                return Dict[args[0], go(args[1])]  # type: ignore
+
         attrs: Optional[Tuple[str, ...]] = None
         constraint_func: Optional[Callable[..., type]] = None
-        if issubclass(annotation, str) and not issubclass(annotation, (EmailStr, DSN, UrlStr, ConstrainedStr)):
-            attrs = ('max_length', 'min_length', 'regex')
-            constraint_func = constr
-        elif lenient_issubclass(annotation, numeric_types) and not issubclass(
-            annotation, (ConstrainedInt, ConstrainedFloat, ConstrainedDecimal, ConstrainedList, bool)
-        ):
-            # Is numeric type
-            attrs = ('gt', 'lt', 'ge', 'le', 'multiple_of')
-            numeric_type = next(t for t in numeric_types if issubclass(annotation, t))  # pragma: no branch
-            constraint_func = _map_types_constraint[numeric_type]
-        elif issubclass(annotation, ConstrainedList):
-            attrs = ('min_items', 'max_items')
-            constraint_func = conlist
+        if isinstance(type_, type):
+            if issubclass(type_, str) and not issubclass(type_, (EmailStr, AnyUrl, ConstrainedStr)):
+                attrs = ('max_length', 'min_length', 'regex')
+                constraint_func = constr
+            elif issubclass(type_, bytes):
+                attrs = ('max_length', 'min_length', 'regex')
+                constraint_func = conbytes
+            elif issubclass(type_, numeric_types) and not issubclass(
+                type_, (ConstrainedInt, ConstrainedFloat, ConstrainedDecimal, ConstrainedList, bool)
+            ):
+                # Is numeric type
+                attrs = ('gt', 'lt', 'ge', 'le', 'multiple_of')
+                numeric_type = next(t for t in numeric_types if issubclass(type_, t))  # pragma: no branch
+                constraint_func = _map_types_constraint[numeric_type]
+
         if attrs:
+            used_constraints.update(set(attrs))
             kwargs = {
                 attr_name: attr
-                for attr_name, attr in ((attr_name, getattr(schema, attr_name)) for attr_name in attrs)
+                for attr_name, attr in ((attr_name, getattr(field_info, attr_name)) for attr_name in attrs)
                 if attr is not None
             }
             if kwargs:
                 constraint_func = cast(Callable[..., type], constraint_func)
                 return constraint_func(**kwargs)
-    return annotation
+        return type_
+
+    ans = go(annotation)
+
+    unused_constraints = constraints - used_constraints
+    if unused_constraints:
+        raise ValueError(
+            f'On field "{field_name}" the following field constraints are set but not enforced: '
+            f'{", ".join(unused_constraints)}. '
+            f'\nFor more details see https://pydantic-docs.helpmanual.io/usage/schema/#unenforced-field-constraints'
+        )
+
+    return ans
 
 
 class SkipField(Exception):
